package file_system;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;

import constants.Constants;

public class Files {
	private String file_id;
	private HashMap<Integer,Chunk> chunk_map;

	public Files(String file_id_temp){
		file_id = file_id_temp;
		chunk_map = new HashMap<Integer,Chunk>();
	}

	public String get_file_id() {
		return file_id;
	}

	public Chunk get_chunk(int chunk_number)
	{
		return chunk_map.get(chunk_number);
	}

	public void add_chunk(Chunk chunk)
	{
		chunk_map.put(chunk.get_chunk_number(), chunk);
	}

	public void delete_chunk(int chunk_number) {

		chunk_map.remove(chunk_number);
	}

	public int size(){
		return chunk_map.size();
	}

	public static Files create_file(File file, int duplication_degree) throws NoSuchAlgorithmException, IOException
	{
		MessageDigest message_digest = MessageDigest.getInstance("SHA-256");
		message_digest.update(file.getAbsolutePath().getBytes());
		byte byte_data[] = message_digest.digest();
		StringBuffer temp_buffer = new StringBuffer();
		for (int i = 0; i < byte_data.length; i++) {
			temp_buffer.append(Integer.toString((byte_data[i] & 0xff) + 0x100, 16).substring(1));
		}
		String file_id = temp_buffer.toString();
		Files return_file = new Files(file_id);
		FileInputStream file_in = new FileInputStream(file);
		int read_temp, chunk_number = 0;
		byte bytes_read[] = new byte[Constants.BODY_MAX_SIZE];
		boolean multiple= true;
		while( (read_temp=file_in.read(bytes_read)) != -1 )
		{
			String data = new String(bytes_read);
			data = data.trim();
			Chunk chunk = new Chunk(file_id,chunk_number,data,duplication_degree);
			return_file.add_chunk(chunk);
			chunk_number++;
			multiple =  (read_temp == Constants.BODY_MAX_SIZE );
			bytes_read = new byte[Constants.BODY_MAX_SIZE];
		}               
		file_in.close();
		if( multiple )
		{
			Chunk chunk = new Chunk(file_id,chunk_number,"",duplication_degree);
			return_file.add_chunk(chunk);
		}
		return return_file;
	}

	public void save_file(String file_id) {
		File_attributes file_attributes = Files_sent.files.get_file_attributes(file_id);
		String file_path = file_attributes.get_file_path();
		try{
			FileWriter file_stream = new FileWriter(file_path);
			BufferedWriter out = new BufferedWriter(file_stream);
			for( int i =0; i < size(); i++)
			{
				Chunk chunk = get_chunk(i);
				if( chunk != null )
					out.write(chunk.get_data());
			}
			out.close();
		}catch (IOException e){
			System.out.println("Error in File.java #01");
		}
	}

}

